@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the Qt Linguist of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
+** a written agreement between you and Digia. For licensing terms and
+** conditions see http://qt.digia.com/licensing. For further information
 ** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** rights. These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -76,8 +68,9 @@ static QString MagicComment(QLatin1String("TRANSLATOR"));
 class FindTrCalls: protected AST::Visitor
 {
 public:
-    FindTrCalls(Engine *engine)
+    FindTrCalls(Engine *engine, ConversionData &cd)
         : engine(engine)
+        , m_cd(cd)
     {
     }
 
@@ -86,7 +79,7 @@ public:
         m_todo = engine->comments();
         m_translator = translator;
         m_fileName = fileName;
-        m_component = QFileInfo(fileName).baseName();
+        m_component = QFileInfo(fileName).completeBaseName();
         accept(node);
 
         // process the trailing comments
@@ -107,8 +100,9 @@ protected:
 
             const QString name = idExpr->name.toString();
             const int identLineNo = idExpr->identifierToken.startLine;
-            if (name == QLatin1String("qsTr") ||
-                name == QLatin1String("QT_TR_NOOP")) {
+            switch (trFunctionAliasManager.trFunctionByName(name)) {
+            case TrFunctionAliasManager::Function_qsTr:
+            case TrFunctionAliasManager::Function_QT_TR_NOOP: {
                 if (!node->arguments) {
                     yyMsg(identLineNo) << qPrintable(LU::tr("%1() requires at least one argument.\n").arg(name));
                     return;
@@ -140,10 +134,11 @@ protected:
                 msg.setExtraComment(extracomment.simplified());
                 msg.setId(msgid);
                 msg.setExtras(extra);
-                m_translator->extend(msg);
+                m_translator->extend(msg, m_cd);
                 consumeComment();
-            } else if (name == QLatin1String("qsTranslate") ||
-                       name == QLatin1String("QT_TRANSLATE_NOOP")) {
+                break; }
+            case TrFunctionAliasManager::Function_qsTranslate:
+            case TrFunctionAliasManager::Function_QT_TRANSLATE_NOOP: {
                 if (! (node->arguments && node->arguments->next)) {
                     yyMsg(identLineNo) << qPrintable(LU::tr("%1() requires at least two arguments.\n").arg(name));
                     return;
@@ -184,10 +179,11 @@ protected:
                 msg.setExtraComment(extracomment.simplified());
                 msg.setId(msgid);
                 msg.setExtras(extra);
-                m_translator->extend(msg);
+                m_translator->extend(msg, m_cd);
                 consumeComment();
-            } else if (name == QLatin1String("qsTrId") ||
-                       name == QLatin1String("QT_TRID_NOOP")) {
+                break; }
+            case TrFunctionAliasManager::Function_qsTrId:
+            case TrFunctionAliasManager::Function_QT_TRID_NOOP: {
                 if (!node->arguments) {
                     yyMsg(identLineNo) << qPrintable(LU::tr("%1() requires at least one argument.\n").arg(name));
                     return;
@@ -213,8 +209,9 @@ protected:
                 msg.setExtraComment(extracomment.simplified());
                 msg.setId(id);
                 msg.setExtras(extra);
-                m_translator->extend(msg);
+                m_translator->extend(msg, m_cd);
                 consumeComment();
+                break; }
             }
         }
     }
@@ -248,6 +245,7 @@ private:
 
     Engine *engine;
     Translator *m_translator;
+    ConversionData &m_cd;
     QString m_fileName;
     QString m_component;
 
@@ -460,7 +458,15 @@ static bool load(Translator &translator, const QString &filename, ConversionData
         return false;
     }
 
-    QString code = QTextStream(&file).readAll();
+    QString code;
+    if (!qmlMode) {
+        code = QTextStream(&file).readAll();
+    } else {
+        QTextStream ts(&file);
+        ts.setCodec("UTF-8");
+        ts.setAutoDetectUnicode(true);
+        code = ts.readAll();
+    }
 
     if (! qmlMode) {
         // fetch the optional pragma directives for Javascript files.
@@ -488,7 +494,7 @@ static bool load(Translator &translator, const QString &filename, ConversionData
     driver.setLexer(&lexer);
 
     if (qmlMode ? parser.parse() : parser.parseProgram()) {
-        FindTrCalls trCalls(&driver);
+        FindTrCalls trCalls(&driver, cd);
 
         //find all tr calls in the code
         trCalls(&translator, filename, parser.rootNode());
