@@ -5,36 +5,28 @@
 **
 ** This file is part of the tools applications of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
+** a written agreement between you and Digia. For licensing terms and
+** conditions see http://qt.digia.com/licensing. For further information
 ** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** rights. These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -495,19 +487,6 @@ void QDBusViewer::refreshChildren()
     model->refresh(tree->currentIndex());
 }
 
-void QDBusViewer::about()
-{
-    QMessageBox box(this);
-
-    box.setText(QString::fromLatin1("<center><img src=\":/qt-project.org/qdbusviewer/images/qdbusviewer-128.png\">"
-                "<h3>%1</h3>"
-                "<p>Version %2</p></center>"
-                "<p>Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).</p>")
-            .arg(tr("D-Bus Viewer")).arg(QLatin1String(QT_VERSION_STR)));
-    box.setWindowTitle(tr("D-Bus Viewer"));
-    box.exec();
-}
-
 void QDBusViewer::anchorClicked(const QUrl &url)
 {
     if (url.scheme() != QLatin1String("qdbus"))
@@ -528,19 +507,3 @@ void QDBusViewer::anchorClicked(const QUrl &url)
     tree->scrollTo(idx);
     tree->setCurrentIndex(idx);
 }
-
-/*!
-  \page qdbusviewer.html
-  \title D-Bus Viewer
-  \keyword qdbusviewer
-
-  The Qt D-Bus Viewer is a tool that lets you introspect D-Bus objects and messages. You can
-  choose between the system bus and the session bus. Click on any service on the list
-  on the left side to see all the exported objects.
-
-  You can invoke methods by double-clicking on them. If a method takes one or more IN parameters,
-  a property editor opens.
-
-  Right-click on a signal to connect to it. All emitted signals including their parameters
-  are output in the message view on the lower side of the window.
-*/
