@@ -5,36 +5,28 @@
 **
 ** This file is part of the tools applications of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
+** a written agreement between you and Digia. For licensing terms and
+** conditions see http://qt.digia.com/licensing. For further information
 ** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** rights. These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -59,9 +51,9 @@ using namespace Microsoft::WRL;
 
 QT_USE_NAMESPACE
 
-#include <ccapi.h>
+#include <ccapi_11.h>
 #include <corecon.h>
-Q_GLOBAL_STATIC(CoreConServer, coreConServer)
+Q_GLOBAL_STATIC_WITH_ARGS(CoreConServer, coreConServer, (11))
 
 #define wchar(str) reinterpret_cast<LPCWSTR>(str.utf16())
 
@@ -250,8 +242,8 @@ bool XapEngine::connect()
     HRESULT hr;
     if (!d->connection) {
         _bstr_t connectionName;
-        hr = coreConServer->handle()->GetConnection(
-                    d->device->handle(), 5000, NULL, connectionName.GetAddress(), &d->connection);
+        hr = static_cast<ICcServer *>(coreConServer->handle())->GetConnection(
+                    static_cast<ICcDevice *>(d->device->handle()), 5000, NULL, connectionName.GetAddress(), &d->connection);
         if (FAILED(hr)) {
             qCWarning(lcWinRtRunner) << "Unable to connect to device." << coreConServer->formatError(hr);
             return false;
